/*
 *     ParallelJ, framework for parallel computing
 *
 *     Copyright (C) 2010 Atos Worldline or third-party contributors as
 *     indicated by the @author tags or express copyright attribution
 *     statements applied by the authors.
 *
 *     This library is free software; you can redistribute it and/or
 *     modify it under the terms of the GNU Lesser General Public
 *     License as published by the Free Software Foundation; either
 *     version 2.1 of the License.
 *
 *     This library is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *     Lesser General Public License for more details.
 *
 *     You should have received a copy of the GNU Lesser General Public
 *     License along with this library; if not, write to the Free Software
 *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

package org.parallelj.internal.kernel.procedure;

import org.parallelj.internal.kernel.KCall;
import org.parallelj.internal.kernel.KProcess;
import org.parallelj.internal.kernel.KProcessor;
import org.parallelj.internal.kernel.KProgram;
import org.parallelj.internal.kernel.KProcedure;
import org.parallelj.internal.util.KProcesses;
import org.parallelj.internal.util.StateEvent;
import org.parallelj.internal.util.StateListener;

public class SubProcessProcedure extends KProcedure {

	/**
	 * The sub program.
	 */
	KProgram subProgram;

	class SubProcessCall extends KCall {

		class SubProcessRunnable implements
				StateListener<KProcess, KProcess.State>, Runnable {

			public SubProcessRunnable() {
			}

			@Override
			public void run() {
				SubProcessCall.this.start();
				KProcess sub = SubProcessProcedure.this.subProgram
						.newProcess(SubProcessCall.this.getContext());
				sub.setParentId(SubProcessCall.this.getId());
				KProcesses.addStateListener(sub, this);
				// TODO check if currentProcessor is not null!
				KProcessor.currentProcessor().submit(sub);
			}

			@Override
			public void stateChanged(StateEvent<KProcess, KProcess.State> event) {
				switch (event.getState()) {
				case ABORTED:
				case TERMINATED:
				case COMPLETED:
					SubProcessCall.this.complete();
				}
			}
		}

		public SubProcessCall(KProcess process) {
			super(SubProcessProcedure.this, process);
		}

		@Override
		public Runnable toRunnable() {
			return new SubProcessRunnable();
		}

	}

	@Override
	protected KCall newCall(KProcess process) {
		return new SubProcessCall(process);
	}

	public SubProcessProcedure(KProgram program) {
		super(program);
	}

	public KProgram getSubProgram() {
		return subProgram;
	}

	public void setSubProgram(KProgram program) {
		this.subProgram = program;
	}

}