/*
 *     ParallelJ, framework for parallel computing
 *
 *     Copyright (C) 2010 Atos Worldline or third-party contributors as
 *     indicated by the @author tags or express copyright attribution
 *     statements applied by the authors.
 *
 *     This library is free software; you can redistribute it and/or
 *     modify it under the terms of the GNU Lesser General Public
 *     License as published by the Free Software Foundation; either
 *     version 2.1 of the License.
 *
 *     This library is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *     Lesser General Public License for more details.
 *
 *     You should have received a copy of the GNU Lesser General Public
 *     License along with this library; if not, write to the Free Software
 *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

package org.parallelj.internal.reflect;

import java.util.ArrayList;
import java.util.List;
import java.util.ServiceLoader;

public abstract class ElementBuilderFactory {

	/*private static ElementBuilderFactory[] builtin = { new JoinFactory(),
			new SplitFactory() };*/

	private static ServiceLoader<ElementBuilderFactory> loader = ServiceLoader
			.load(ElementBuilderFactory.class);

	public abstract ElementBuilder newBuilder(Class<?> type);

	public static ElementBuilder[] newBuilders(Class<?> type) {

		List<ElementBuilder> builders = new ArrayList<ElementBuilder>();

		/*for (ElementBuilderFactory factory : builtin) {
			ElementBuilder b = factory.newBuilder(type);
			if (b != null) {
				builders.add(b);
			}
		}*/
		for (ElementBuilderFactory factory : loader) {
			ElementBuilder b = factory.newBuilder(type);
			if (b != null) {
				builders.add(b);
			}
		}
		return builders.toArray(new ElementBuilder[0]);
	}
}