/*
 *     ParallelJ, framework for parallel computing
 *
 *     Copyright (C) 2010 Atos Worldline or third-party contributors as
 *     indicated by the @author tags or express copyright attribution
 *     statements applied by the authors.
 *
 *     This library is free software; you can redistribute it and/or
 *     modify it under the terms of the GNU Lesser General Public
 *     License as published by the Free Software Foundation; either
 *     version 2.1 of the License.
 *
 *     This library is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *     Lesser General Public License for more details.
 *
 *     You should have received a copy of the GNU Lesser General Public
 *     License along with this library; if not, write to the Free Software
 *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

package org.parallelj.internal.util;

import org.parallelj.internal.kernel.KCall;
import org.parallelj.internal.kernel.KCall.State;

/**
 * Classes that provides helper methods for {@link KCall}.
 * 
 * @author Atos Worldline
 * 
 */
public privileged aspect KCalls {

	/**
	 * Property that will store the listeners.
	 * 
	 * @return
	 */
	StateChangeSupport<KCall, KCall.State> KCall.support;

	/**
	 * Advice that create the StateChangeSupport once a KCall is
	 * Instantiated.
	 * 
	 * @param self
	 */
	after(KCall self) : execution(KCall.new(..)) && this(self)  {
		self.support = new StateChangeSupport<KCall, State>(self);
	}

	Object around(KCall self) : call (* KCall.State.*(KCall)) && args(self) {
		KCall.State state = self.state;
		Object object = null;
		self.lock();
		try  {
			object = proceed(self);
		} finally {
			self.unlock();
		}
		if (self.state != state) {
			this.stateChanged(self, state, self.state);
		}
		return object;
	}
	
	protected void stateChanged(KCall self, State oldState, State newState) {
		self.support.fireStateChanded(self.state);
	}

	public static void addStateListener(KCall target,
			StateListener<KCall, State> listener) {
		target.support.addStateListener(listener);
	}

	public void removeStateListener(KCall target,
			StateListener<KCall, State> listener) {
		target.support.removeStateListener(listener);
	}

}
